--[[
ClosetGnome, a World of Warcraft addon to manage item sets.
Copyright (C) 2007 Rabbit.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

local ClosetGnome = ClosetGnome
assert(ClosetGnome, "ClosetGnome-UI-FuBar requires ClosetGnome to run.")

-----------------------------------------------------------------------
-- Localization
local L = AceLibrary("AceLocale-2.2"):new("ClosetGnome-UI-FuBar")

L:RegisterTranslations("enUS", function() return {
	["Menu"] = true,
	["Menu options."] = true,
	["Available sets"] = true,
	["%s |cffd9d919*|r"] = true,
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = true,
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = true,
	["Use the options menu to equip, edit, add or delete item sets."] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	["Available sets"] = "Verfgbare Sets:",
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = "|cffeda55fKlicke|r ein Set an um die Ausrstung zu wechseln. |cffeda55fStrg-Klick|r um es anzuprobieren. |cffeda55fShift-Klicke|r ein Set um es zu editieren. Das zuletzt benutzte Set ist mit einem Stern gekennzeichnet.",
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = "Es wurden noch keine Ausrstungssets erstellt. Klicke mit der rechten Maustaste auf das Icon, dann 'Hinzufgen' um ein Set zu erstellen",
} end)

L:RegisterTranslations("esES", function() return {
	["Available sets"] = "Conjuntos disponibles",
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = "Haz |cffeda55fClic|r en un conjunto de arriba para cambiar tu equipo. |cffeda55fCtrl-Clic|r para probrtelo. |cffeda55fMaysculas-Clic|r para editarlo. El ltimo conjunto equipado est marcado con un asterisco.",
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = "Todav\195\173a no has creado ning\195\186n conjunto de equipo. Haz |cffeda55fDoble-Clic|r para crear uno.",
} end)

L:RegisterTranslations("frFR", function() return {
	["Available sets"] = "Ensembles disponibles",
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = "|cffeda55fCliquez|r sur un ensemble ci-dessus pour changer votre \195\169quipement. |cffeda55fCtrl-clic|r sur un ensemble pour l'essayer. |cffeda55fShift-Clic|r sur un ensemble pour l'\195\169diter. L'ensemble \195\169quip\195\169 en dernier est indiqu\195\169 par une \195\169toile.",
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = "Vous n'avez pas encore d'ensembles. Faites un |cffeda55fdouble-clic|r pour cr\195\169er un ensemble.",
} end)

L:RegisterTranslations("koKR", function() return {
	["Available sets"] = "??? ??",
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = "??? ????? ??? |cffeda55f??|r???. ??? ??? ??? |cffeda55fCtrl-??|r???. ??? ????? |cffeda55fShift-??|r???. ??? ?? ??? ? ??? ?????.",
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = "??? ?? ??? ?? ???? ?????. ??? ????? |cffeda55f??-??|r???.",
} end)

L:RegisterTranslations("zhCN", function() return {
	["Available sets"] = "????",
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = "|cffeda55f??|r????????, |cffeda55f<Ctrl>+??|r??????, |cffeda55f<Shift>+??|r????, ???????????.",
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = "???????????, ?|cffeda55f??|r??.",
} end)

L:RegisterTranslations("zhTW", function() return {
	["Available sets"] = "????",
	["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."] = "\n|cffeda55f??: |r?????\n|cffeda55fCtrl-??: |r?????\n|cffeda55fShift-??: |r?????\n???????????????",
	["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."] = "?????????,|cffeda55f????|r???????",
} end)

-----------------------------------------------------------------------
-- FuBar UI for ClosetGnome

local tablet = AceLibrary:HasInstance("Tablet-2.0") and AceLibrary("Tablet-2.0") or nil
local defaultIcon = "Interface\\Icons\\INV_Chest_Cloth_17"

CGFu = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "FuBarPlugin-2.0", "AceEvent-2.0")
local CGFu = CGFu
CGFu.hasIcon = true

local lastEquippedSet = nil
local sortedSets = {}

local function updateSortedSets()
	for i in ipairs(sortedSets) do sortedSets[i] = nil end
	for k in pairs(ClosetGnome.db.char.set) do table.insert(sortedSets, k) end
	table.sort(sortedSets)
end

function CGFu:OnInitialize()
	self:RegisterDB("ClosetGnomeDB", "ClosetGnomePerCharDB")

	local args = AceLibrary("FuBarPlugin-2.0"):GetAceOptionsDataTable(self)
	local options = ClosetGnome.options
	if not options.args[L["Menu"]] then
		options.args[L["Menu"]] = {
			type = "group",
			name = L["Menu"],
			desc = L["Menu options."],
			args = args,
			order = 300,
		}
	end

	self.hideWithoutStandby = true
	self.hasIcon = defaultIcon
	self.hasNoColor = true
	self.defaultMinimapPosition = 240
	self.clickableTooltip  = true
	self.OnMenuRequest = options

	if not tablet then
		self.blizzardTooltip = true
	end
end

function CGFu:OnEnable()
	local function updateLatest(set)
		lastEquippedSet = set
		updateSortedSets()
		self:UpdateDisplay()
	end
	local function removeLatest(set)
		if lastEquippedSet == set then
			lastEquippedSet = nil
		end
		updateSortedSets()
		self:UpdateDisplay()
	end
	self:RegisterEvent("ClosetGnome_UpdateSet", updateLatest)
	self:RegisterEvent("ClosetGnome_AddSet", updateLatest)
	self:RegisterEvent("ClosetGnome_WearSet", updateLatest)
	self:RegisterEvent("ClosetGnome_DeleteSet", removeLatest)
	self:RegisterEvent("ClosetGnome_PartlyWearSet", "UpdateDisplay")
	self:RegisterEvent("ClosetGnome_KeybindingsUpdated", "UpdateDisplay")

	if AceLibrary("AceEvent-2.0"):IsFullyInitialized() then
		self:AceEvent_FullyInitialized()
	end
	self:RegisterEvent("AceEvent_FullyInitialized")

	updateSortedSets()
end

function CGFu:AceEvent_FullyInitialized()
	-- Loop through the current sets and check if we have any of them equipped.
	for k,v in pairs(ClosetGnome.db.char.set) do
		local ret = ClosetGnome:IsSetFullyEquipped(k)
		if ret then
			lastEquippedSet = k
			ClosetGnome:TriggerEvent("ClosetGnome_WearSet", k)
			break
		end
	end
	self:UpdateDisplay()
end

function CGFu:OnTextUpdate()
	if lastEquippedSet then
		self:SetText(ClosetGnome:GetSetDisplayName(lastEquippedSet))
	else
		self:SetText("ClosetGnome")
	end
	if self.db.char.icons[lastEquippedSet] then
		self:SetIcon(ClosetGnome.db.char.icons[lastEquippedSet])
	else
		self:SetIcon(defaultIcon)
	end
end

function CGFu:OnDoubleClick()
	if IsShiftKeyDown() then
		ClosetGnome:CreateOrUpdateSet(lastEquippedSet)
	else
		ClosetGnome:CreateOrUpdateSet()
	end
end

local function clickfunc(name)
	if IsControlKeyDown() then
		local set = ClosetGnome.db.char.set[name]
		DressUpModel:Undress()
		for i = 1, 19 do
			local item = set[i]
			if i ~= 18 and item ~= false then
				DressUpItemLink(item)
			end
		end
	elseif IsShiftKeyDown() then
		ClosetGnome:CreateOrUpdateSet(name)
	else
		ClosetGnome:WearSet(name)
	end
end

local function blizzardTooltip()
	GameTooltip:AddLine("ClosetGnome")
	if #sortedSets > 0 then
		GameTooltip:AddLine(L["Available sets"], 1, 1, 1)
		for i, set in ipairs(sortedSets) do
			local displayName = ClosetGnome:GetSetDisplayName(set)
			if lastEquippedSet == set then
				displayName = L["%s |cffd9d919*|r"]:format(ClosetGnome:GetSetDisplayName(set))
			end
			GameTooltip:AddDoubleLine(
				displayName,
				ClosetGnome.db.char.keybindings[set] or "",
				0.7, 1, 0.2,
				0.5, 0.5, 0.5
			)
		end
		GameTooltip:AddLine(L["Use the options menu to equip, edit, add or delete item sets."], 0.2, 1, 0.2, 1)
	else
		GameTooltip:AddLine(L["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."], 0.2, 1, 0.2, 1)
	end
end

local function tabletTooltip()
	local cat = nil
	for i, set in ipairs(sortedSets) do
		if not ClosetGnome:HideSetFromUI(set) then
			if not cat then
				cat = tablet:AddCategory(
					"columns", 2,
					"text", L["Available sets"],
					"showWithoutChildren", false,
					"child_hasCheck", true,
					"child_checked", true,
					"child_text2R", 0.5,
					"child_text2G", 0.5,
					"child_text2B", 0.5
				)
			end
			local displayName = ClosetGnome:GetSetDisplayName(set)
			if lastEquippedSet == set then
				displayName = L["%s |cffd9d919*|r"]:format(ClosetGnome:GetSetDisplayName(set))
			end
			cat:AddLine(
				"text", displayName,
				"func", clickfunc,
				"arg1", set,
				"checkIcon", ClosetGnome.db.char.icons[set] or "",
				"text2", ClosetGnome.db.char.keybindings[set] or ""
			)
		end
	end

	if cat ~= nil then
		tablet:SetHint(L["|cffeda55fClick|r a set above to change your equipment. |cffeda55fCtrl-Click|r a set to try it on. |cffeda55fShift-Click|r a set to edit it. The last equipped set is indicated by a star."])
	else
		tablet:SetHint(L["You have not created any equipment sets yet. |cffeda55fDouble-Click|r to create a set."])
	end
end

function CGFu:OnTooltipUpdate()
	if self.blizzardTooltip then
		blizzardTooltip()
	else
		tabletTooltip()
	end
end

